package fr.mrdev023.vulkan_java.vk;

import static org.lwjgl.vulkan.VK10.vkGetDeviceQueue;
import static org.lwjgl.vulkan.VK10.vkQueueWaitIdle;

import org.lwjgl.PointerBuffer;
import org.lwjgl.system.MemoryStack;
import org.lwjgl.vulkan.VkQueue;
import org.tinylog.Logger;

public class Queue {

    private final VkQueue vkQueue;
    private final int familyIndex;
    private final int queueIndex;

    public Queue(Device device, int queueFamilyIndex, int queueIndex) {
        this.familyIndex = queueFamilyIndex;
        this.queueIndex = queueIndex;

        try (MemoryStack stack = MemoryStack.stackPush()) {
            PointerBuffer pQueue = stack.mallocPointer(1);
            vkGetDeviceQueue(device.getVkDevice(), queueFamilyIndex, queueIndex, pQueue);
            long queue = pQueue.get(0);
            vkQueue = new VkQueue(queue, device.getVkDevice());
        }

        Logger.debug("Queue created on family index {} and queue index {}", familyIndex, queueIndex);
    }

    public VkQueue getVkQueue() {
        return vkQueue;
    }

    public int getFamilyIndex() {
        return familyIndex;
    }

    public int getQueueIndex() {
        return queueIndex;
    }

    public void waitIdle() {
        vkQueueWaitIdle(vkQueue);
    }
}