package fr.mrdev023.vulkan_java.vk;

import java.nio.FloatBuffer;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.lwjgl.system.MemoryStack;
import org.lwjgl.vulkan.VK10;
import org.lwjgl.vulkan.VkDeviceQueueCreateInfo;

public class QueuesCreator {
    private final HashMap<Integer, List<Float>> families;
    private HashMap<Integer, Integer> counters;

    public QueuesCreator(QueueFamilyIndices queueFamilyIndices) {
        this.families = createFamilies(queueFamilyIndices);
        this.counters = new HashMap<>();
        for (var family : families.keySet()) {
            counters.put(family, 0);
        }
    }

    public VkDeviceQueueCreateInfo.Buffer createQueueCreateInfos(MemoryStack stack,
            PhysicalDevice physicalDevice) {

        VkDeviceQueueCreateInfo.Buffer queueCreationInfoBuf = VkDeviceQueueCreateInfo.calloc(families.size(),
                stack);
        int i = 0;
        for (var familyIndex : families.keySet()) {
            FloatBuffer priorities = stack.callocFloat(families.get(familyIndex).size());
            for (var priority : families.get(familyIndex)) {
                priorities.put(priority);
            }
            priorities.flip();
            queueCreationInfoBuf.get(i)
                    .sType(VK10.VK_STRUCTURE_TYPE_DEVICE_QUEUE_CREATE_INFO)
                    .queueFamilyIndex(familyIndex)
                    .pQueuePriorities(priorities);
            i++;
        }

        return queueCreationInfoBuf;
    }

    public Queue createQueue(Device device, int queueFamilyIndex) {
        var queueIndex = getNextAvailableQueueIndexForFamilyIndex(queueFamilyIndex);
        return new Queue(device, queueFamilyIndex, queueIndex);
    }

    private int getNextAvailableQueueIndexForFamilyIndex(int queueFamilyIndex) {
        var max = families.get(queueFamilyIndex).size();
        var current = counters.getOrDefault(queueFamilyIndex, 0);
        if (current >= max) {
            throw new RuntimeException("No more queues available for family " + queueFamilyIndex);
        }

        counters.put(queueFamilyIndex, current + 1);
        return current;
    }

    private HashMap<Integer, List<Float>> createFamilies(QueueFamilyIndices queueFamilyIndices) {
        var families = new HashMap<Integer, List<Float>>();
        if (queueFamilyIndices.getGraphicsQueueFamilyIndex().isPresent()) {
            var familyIndex = queueFamilyIndices.getGraphicsQueueFamilyIndex().get();
            var familyPriorities = families.getOrDefault(familyIndex, new ArrayList<Float>());
            familyPriorities.add(1.0f);
            families.put(familyIndex, familyPriorities);
        }
        if (queueFamilyIndices.getComputeQueueFamilyIndex().isPresent()) {
            var familyIndex = queueFamilyIndices.getComputeQueueFamilyIndex().get();
            var familyPriorities = families.getOrDefault(familyIndex, new ArrayList<Float>());
            familyPriorities.add(0.5f);
            families.put(familyIndex, familyPriorities);
        }
        if (queueFamilyIndices.getTransferQueueFamilyIndex().isPresent()) {
            var familyIndex = queueFamilyIndices.getTransferQueueFamilyIndex().get();
            var familyPriorities = families.getOrDefault(familyIndex, new ArrayList<Float>());
            familyPriorities.add(0.5f);
            families.put(familyIndex, familyPriorities);
        }
        return families;
    }
}