package fr.mrdev023.vulkan_java.vk;

import static org.lwjgl.glfw.GLFWVulkan.glfwVulkanSupported;

import java.util.Optional;
import java.util.Set;

import org.lwjgl.vulkan.KHRCreateRenderpass2;
import org.lwjgl.vulkan.KHRDepthStencilResolve;
import org.lwjgl.vulkan.KHRDynamicRendering;
import org.lwjgl.vulkan.KHRMaintenance2;
import org.lwjgl.vulkan.KHRMultiview;
import org.lwjgl.vulkan.KHRPortabilitySubset;
import org.lwjgl.vulkan.KHRSwapchain;
import org.lwjgl.vulkan.VK10;

import fr.mrdev023.vulkan_java.vk.validators.PhysicalDeviceCompatibilityValidator;
import fr.mrdev023.vulkan_java.window.Display;

public class Vulkan {

    private static Instance instance;
    private static PhysicalDevice physicalDevice;
    private static Device device;
    private static Surface surface;
    private static Swapchain swapchain;

    public static void init() throws VulkanError {
        if (!glfwVulkanSupported()) {
            throw new IllegalStateException("Cannot find a compatible Vulkan installable client driver (ICD)");
        }

        instance = new Instance(true);
        surface = new Surface(instance, Display.getWindow());

        var extensions = Set.of(KHRDynamicRendering.VK_KHR_DYNAMIC_RENDERING_EXTENSION_NAME,
                // Dependency of KHRDynamicRendering
                KHRDepthStencilResolve.VK_KHR_DEPTH_STENCIL_RESOLVE_EXTENSION_NAME,
                // Dependency of KHRDepthStencilResolve
                KHRCreateRenderpass2.VK_KHR_CREATE_RENDERPASS_2_EXTENSION_NAME,
                // Dependency of KHRCreateRenderpass2
                KHRMaintenance2.VK_KHR_MAINTENANCE_2_EXTENSION_NAME,
                KHRMultiview.VK_KHR_MULTIVIEW_EXTENSION_NAME,
                // Required to create swapchain
                KHRSwapchain.VK_KHR_SWAPCHAIN_EXTENSION_NAME);
        if (VulkanUtils.getOS() == VulkanUtils.OSType.MACOS) {
            extensions.add(KHRPortabilitySubset.VK_KHR_PORTABILITY_SUBSET_EXTENSION_NAME);
        }

        var physicalDeviceCompatibilityValidator = new PhysicalDeviceCompatibilityValidator(
                VK10.VK_QUEUE_GRAPHICS_BIT | VK10.VK_QUEUE_COMPUTE_BIT | VK10.VK_QUEUE_TRANSFER_BIT,
                extensions,
                Optional.of(surface));

        var physicalDeviceMatch = SuitablePhysicalDeviceFinder.findBestPhysicalDevice(instance,
                physicalDeviceCompatibilityValidator);
        if (physicalDeviceMatch == null) {
            throw new RuntimeException("No suitable physical device found");
        }

        physicalDevice = physicalDeviceMatch.physicalDevice;
        device = new Device(physicalDeviceMatch);
        swapchain = new Swapchain(device, surface);
    }

    public static void destroy() {
        swapchain.destroy();
        surface.destroy();
        device.destroy();
        physicalDevice.destroy();
        instance.destroy();
    }

}
